function [thetahat,yhat,Phat] = rlsff(YU,nk,lambda)
%
% La funzione [thetahat,yhat] = rlsff(YU,nk,lambda) implementa 
% l'algoritmo dei minimi quadrati ricorsivi con fattore d'oblio, 
% lambda, una volta definito l'ordine del modello ipotizzato, nk.
% La matrice YU contiene il vettore dell'uscita affiancato a quello 
% degli eventuali ingressi. Viene restituita la matrice thetahat delle 
% stime dei parametri del modello, ed il vettore con le previsioni
% dell'uscita, yhat.
%


rho = 100; % Varianza della stima iniziale di thetahat(0), P(0) = rho * I;

if(nargin < 3), lambda = 1; end; % RLS senza forgetting factor

[N,nio] = size(YU); % N campioni e almeno una colonna, altrimenti 1 + numero di ingressi
                    % nio >= 1

thetahat = zeros(nk*nio,N); % thetahat(0) = 0
yhat = zeros(N,1);          % Modello MISO, una sola uscita
Phat = zeros(nk*nio,N);     % Elementi sulla diagonale della matrice di covarianza

Pt = rho * eye(nk*nio);

for t = nk+1:N,
    if(nio == 1), psit = YU(t-1:-1:t-nk,1)'; end; % solo l'uscita: modello AR
    
    if(nio == 2), psit = [YU(t-1:-1:t-nk,1)' YU(t-1:-1:t-nk,2)']'; end; % modello ARX SISO
    
    if(nio > 2),  for indxi = 1:nio, psit = [psit' YU(t-1:-1:t-nk,indxi)']';
                      psit = [YU(t-1:-1:t-nk,1)' psit']'; % modello ARX MISO
                  end;
    end;
    
    yhat(t,1) = (psit')*thetahat(:,t-1);
    
    epst = YU(t,1) - yhat(t,1);
    
    Pt = (1/lambda)*(Pt - (Pt*psit*(psit')*Pt)/(lambda + (psit')*Pt*psit));
    
    Kt = Pt * psit;
    
    thetahat(:,t) = thetahat(:,t-1) + Kt * epst; 
    
    Phat(:,t) = diag(Pt); 
end

return